lappend auto_path .
package require nx 
package require STORM
################################
#####simple example Account#####
################################

nx::Class create Customer {
	:property -accessor public {name}
	:property -accessor public {accounts ""}
	
	:public method addAccount {account:object,type=SavingsAccount} {
		:accounts set [linsert [:accounts get] 0 $account ]	
		$account owner set [self]
	}
	
	:public method openAccount {accountType} {
		if { $accountType eq "savings" } {
			
			set s [SavingsAccount new ]
			:addAccount $s 
			return $s
			
		} elseif { $accountType eq "checking" } {
			set s [CheckingAccount new]
			:addAccount $s
			return $s
			
		} elseif { $accountType eq "openA" } {
			set s [OpenAccount new]
			:addAccount $s
			return $s
			
		} else { puts Error }	
	}
	
	:public method cancelAccount {account:object,type=SavingsAccount} {
		foreach a [:accounts get] {
			if {$account eq $a} {
				#pop account
				puts accountFound
				$account destroy
			} else {
				puts accountNotFound
			}			
		}				
	}	
}

nx::Class create SavingsAccount {
	:property -accessor public {number:integer}
	:property -accessor public {balance 0.0}
	:property -accessor public {owner:object,type=Customer}
	:property -accessor public {balancing:object,type=Balancing}
			
	:public method deposit {amount:double} {
		:balance set [expr {[:balance get] + $amount}]
	}
	
	:public method withdraw {amount:double} {
		:balance set [expr {[:balance get] - $amount}]
	}	
}

nx::Class create CheckingAccount -superclass SavingsAccount {
	
	:public method transfer {amount:double, toAccount:object,type=SavingsAccount} {
		if {[expr {[:balance get] - $amount}] < 0} {
			puts tooLessMoneyForTransfer
			
		} elseif {[expr {[:balance get] - $amount}] >= 0} {		
			:balance set [expr {[:balance get] - $amount}]
			$toAccount balance set [expr {[$toAccount balance get] + $amount} ]
		}
	}
	:public method withdraw {amount:double} {
		if {[expr {[:balance get] - $amount}] < 0} {
			puts tooLessMoneyForWithdraw
						
		} elseif {[expr {[:balance get] - $amount}] >= 0} {		
			:balance set [expr {[:balance get] - $amount}]
		}	
	}
}

nx::Class create OpenAccount -superclass CheckingAccount {
	:property -accessor public {limit:double 0.0}

	:public method transfer {amount:double, toAccount:object,type=SavingsAccount} {	
		if {[expr {[:balance get] - $amount}] < [:limit get]} {
			puts tooLessMoneyForTransfer
			
		} elseif {[expr {[:balance get] - $amount}] >= [:limit get]} {		
			:balance set [expr {[:balance get] - $amount}]
			$toAccount balance set [expr {[$toAccount balance get] + $amount} ]
		}
	}
	:public method withdraw {amount:double} {
		if {[expr {[:balance get] - $amount}] < [:limit get]} {
			puts tooLessMoneyForWithdraw
							
		} elseif {[expr {[:balance get] - $amount}] >= [:limit get]} {		
			:balance set [[:balance get] - $amount]	
		}				
	}	
}

nx::Class create Balancing {
	:property -accessor public {rateOfInterest:double} 

	:public method close {} {	
	}
}
nx::Class create CalendarYear -superclass Balancing {
	:property -accessor public {rateOfInterest:double 0.5}
	:public method close {} {	
	}
}
nx::Class create Month -superclass Balancing {
	:property -accessor public {rateOfInterest:double 0.1}
	:public method close {} {	
	}
}
nx::Class create Quarter -superclass Balancing {
	:property -accessor public {rateOfInterest:double 0.2}
	:public method close {} {	
	}
}


##scenario-test specification
#TestCase AccountOpening
set ao [::STORM::TestCase new -name AccountOpening]
$ao setup_script set {set c [Customer new]}
$ao preconditions set { {expr {[llength [::Customer info instances]] == 1}} } 

$ao postconditions set { {expr {[llength [::SavingsAccount info instances]] == 0}}
  {expr {[llength [[::Customer info instances] accounts get] ] == 1}}  }
$ao cleanup_script set {[::Customer info instances] destroy }

#TestScenario SavingsAccOpening
set osa [::STORM::TestScenario new -name SavingsAccOpening -testcase AccountOpening]
$osa preconditions set { {expr {[llength [::SavingsAccount info instances]] == 0}} }
$osa test_body set {[::Customer info instances] openAccount savings}
$osa postconditions set {{expr {[llength [::SavingsAccount info instances]] == 1}}}
$osa cleanup_script set {[::SavingsAccount info instances] destroy}
#TestScenario OpenCheckingAcc
#TestScenario OpenAcc  

#TestCase Withdrawal
set wd [::STORM::TestCase new -name Withdrawal]
$wd setup_script set { set c [Customer new] }
$wd preconditions set { {expr {[llength [::Customer info instances]] == 1}} }
$wd postconditions set { }
$wd cleanup_script set {}

#TestScenario WithdrawFromChecking
set wdc [::STORM::TestScenario new -name WithdrawFromChecking -testcase Withdrawal]
$wdc setup_script set { 
  set cha [CheckingAccount new] 
  $cha balance set 500.0  
}
$wdc preconditions set { {expr {[llength [::CheckingAccount info instances]] == 1}}}
$wdc test_body set { 
  [::CheckingAccount info instances] withdraw 200
  return [[::CheckingAccount info instances] balance get]
}
$wdc expected_result set 300.0
$wdc postconditions set { }

#TestCase Deposition
set dp [::STORM::TestCase new -name Deposition]
$dp setup_script set { }
$dp preconditions set { }
$dp postconditions set { }
$dp cleanup_script set { }

#TestCase Transfer
set tf [::STORM::TestCase new -name Transfer]
$tf setup_script set { } 
$tf preconditions set { }
$tf postconditions set { }
$tf cleanup_script set { }


##run test in STORM
set AccountTests  [::STORM::TestSuite new -detailed_report 1]
$AccountTests order set [lsort -unique [::STORM::TestCase info instances]]
$AccountTests runAllTestCases

##create with config model all call types and isEntireTestSuite
set vmc [::VM::VMCreator new]
$vmc createXMIModel view1 both "" "" "" true
