package require nx 
package provide VM 0.1
#####################################################################
#
# KaleidoScope - View Model Creator
#
#####################################################################
namespace eval ::VM {
  nx::Class create VMCreator {
    :public method createXMIModel {vName callScope testCase testScenario block isEntireTS} {
      set partition [::VM::TestPartition new -name "p1" -testCaseName $testCase -testScenarioName $testScenario -testBlockName $block -isEntireTS $isEntireTS]
      set view [::VM::View new -name $vName -callScope $callScope -partition $partition]
      set xmiText [:buildHeader]
      append xmiText [:buildView $view]
      set path [file join [pwd] [$view name get].xmi]
      set f $path
      set fileId [open $f "w"]
      puts -nonewline $fileId $xmiText
      close $fileId
      puts "XMI view model **[$view name get]** can be found under $path"
    }
    :public method buildHeader {} {
      return "<?xml version='1.0' encoding='ASCII'?>"
    }
    :public method buildView {view} {
      set viewTag "<VM:View xmi:version='2.0' xmlns:xmi='http://www.omg.org/XMI' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:VM='viewModel' xsi:schemaLocation='viewModel VIEW.ecore'"
      append viewTag " callScope='[$view callScope get]' name='[$view name get]'>"
      append viewTag [:buildPartition [$view partition get]]
      append viewTag " </VM:View>"
      return $viewTag
    }
    :public method buildPartition {partition} {
      set partitionTag " <partition "
      if {[$partition testBlockName get] eq "null" || [$partition testBlockName get] eq ""} {
      } else {
	append partitionTag "testBlock='[$partition testBlockName get]' "	
      }
      if {[$partition testScenarioName get] eq "null" || [$partition testScenarioName get] eq ""} {
      } else {
	append partitionTag "testScenario='[$partition testScenarioName get]' "	
      }
      if {[$partition testCaseName get] eq "null" || [$partition testCaseName get] eq ""} {
      } else {
	append partitionTag "testCase='[$partition testCaseName get]' "	
      }
      append partitionTag  "isEntireTestSuite='[$partition isEntireTS get]' name='[$partition name get]'/>"	
      return $partitionTag
    }
  } 	
  nx::Class create View { 	
    :property -accessor public {name}
    :property -accessor public {callScope}
    :property -accessor public {partition}
  }
  nx::Class create TestPartition { 
    :property -accessor public {name}
    :property -accessor public {testCaseName}
    :property -accessor public {testScenarioName}
    :property -accessor public {testBlockName}
    :property -accessor public {isEntireTS}
  }

  
}
